import tkinter as tk

class Task(tk.Frame):
	"""Il s'agit de la classe définissant une tâche
	   Elle permet son affichage ainsi que celui de ses tâches enfants
	   Chaque tâche a un identifiant, elle a aussi l'identifiant des sous tâches ainsi que de la tâche parent"""
	   
	#variable statique permettant l'assignation d'un identifiant unique à chaque tâche
	idCounter = 0
	
	def __init__(self, masterId, tree, topLevel = False):
		"""Constructeur de la classe Task"""
		
		tk.Frame.__init__(self, tree.allTasks[masterId])
		self.parentid = masterId #représente l'id du parent
		self.tree = tree #représente le tasktree
		self.tree.allTasks[Task.idCounter] = self #assigne la tâche courante au tasktree
		self.row = 0 #row représente la ligne à laquelle se trouve la tâche dans le grid_manager du parent
						 #par défaut elle vaut 0, elle est réassigné par le parent directement
		
		self.level=self.tree.allTasks[self.parentid].level + 1 #level de la tâche dans l'arbre
		self.id = Task.idCounter #on assigne l'id unique et on incrémente le compteur
		Task.idCounter+=1
		
		#Configuration de la grid de la tâche courante
		self.columnconfigure(0,weight=1)
		
		#il s'agit des infos de la tâche de ce niveau
		self.boutonCacher = tk.Button(self, text="Hide", command=self._hideButton) #bouton permettant de cacher les sous-tâches
		self.boutonCacher.grid(row = 1, sticky = tk.W, pady=(0,10)) #on affiche le bouton
		self.hidden = False #les sous tâches sont visibles par défaut
		self.selected = False #la tâche courante est déselectionnée par défaut
		
		#la mainTask est la tâche courante, c'est réellement ici qu'on affiche la tâche de ce niveau
		self.mainTask = tk.Frame(self, bg = "grey", height = 26) #contient les données de la tâche principale
		#quelques configuration de la grid de la mainTask
		self.mainTask.grid_propagate(0) #permet d'éviter le redimensionnement de ce widget, sa taille ne dépend plus des widgets enfants
		self.mainTask.columnconfigure(3,weight=1) 
		self.mainTask.columnconfigure(1,minsize=40) 
		self.mainTask.grid(sticky=tk.W+tk.E, row = 0) 
		
		#widgets de la mainTask
		self.name = tk.Label(self.mainTask, text = str(self.id)) 
		self.name.grid(padx = 30, column = 0, sticky = tk.W)
		self.boutonSupprimer = tk.Button(self.mainTask, text="Delete") #bouton de suppression de la tâche
		self.boutonSupprimer.grid(column = 3, sticky = tk.E, padx=(0,30), row = 0)
		self.boutonSupprimer["command"]=self._delete
		self.boutonNouveau = tk.Button(self.mainTask, text="New", command=self._newTask) #bouton de création de tâche enfant
		
		#les sous tâches sous forme de liste de leur id
		self.subtasks = []
		
		#les events
		self.mainTask.bind("<Enter>", self._enterMainTask)
		self.mainTask.bind("<Leave>", self._leaveMainTask)
		self.bind("<Enter>", self._enterWholeTask)
		self.bind("<Leave>", self._leaveWholeTask)
		self.mainTask.bind("<Button-1>", self._clicked)
		self.mainTask.bind("<Shift-Button-1>", self._alsoClicked)
		
		#test -----------------------------------------------------------
		self.boutonDico = tk.Button(self.mainTask, text="DicoTest", command=self._dicoTest)
		self.boutonDico.grid(column = 2, sticky = tk.W, row = 0)
		#----------------------------------------------------------------
		
	def _dicoTest(self):
		print("nombre de taches :", len(self.tree.allTasks.items()))
		
		
	def addSubtask(self, task):
		"""Ajout du sous-tache à celle-ci"""
		
		task.grid(sticky=tk.N+tk.S+tk.W+tk.E, row=2, padx=(30,0))
		task.row = 2
		for index, subt in enumerate(self.subtasks): #on réajuste les sous tâche pour que la nouvelle soit tout au dessus
			self.tree.allTasks[subt].row += 1
			self.tree.allTasks[subt].grid(sticky=tk.N+tk.S+tk.W+tk.E, row=self.tree.allTasks[subt].row, padx=(30,0))
			
		self.subtasks.append(task.id)
		
		if self.hidden:
			self._hideSubtasks()
		
		
	def _delete(self):
		"""destruction de la tâche courante, qui est déréférencée de son parent"""
		
		if self.parentid != -1:
			self.tree.allTasks[self.parentid].subtasks.remove(self.id)
		self.tree.deleteTask(self.id)
		
		
	def _enterMainTask(self, event):
		"""Callback appelé lorsque la souris entre sur la tâche de ce niveau (mainTask)"""
		
		if not self.selected:
			self.mainTask["bg"] = "#f77"
		
		
	def _leaveMainTask(self, event):
		"""Callback appelé lorsque la souris quitte la tâche de ce niveau (mainTask)"""
		
		if not self.selected:
			self.mainTask["bg"] = "grey"
		
		
	def _enterWholeTask(self, event):
		"""Callback appelé lorsque la souris entre sur la tâche ou une de ses enfants"""
		
		self.boutonNouveau.grid(column = 1, row = 0)
		
		
	def _leaveWholeTask(self, event):
		"""Callback appelé lorsque la souris quitte la tâche ou une de ses enfants"""
		
		self.boutonNouveau.grid_remove()
		
		
	def _showSubtasks(self):
		""" on affiche les sous tâche qui avaient été cachées """
		
		for id in self.subtasks:
			self.tree.allTasks[id].grid()
		
	
	def _hideSubtasks(self):
		""" on cache les sous taches """
		
		for id in self.subtasks:
			self.tree.allTasks[id].grid_remove()
			
			
	def _hideButton(self):
		""" callback appelé lors du click sur le bouton cacher """
	
		if self.hidden:
			self.hidden = False
			self._showSubtasks()
			self.boutonCacher["text"]="hide"
		else:
			self.hidden = True
			self._hideSubtasks()
			self.boutonCacher["text"]="show"
			
	def _newTask(self):
		"""Cette fonction ajoute une nouvelle tâche enfant"""
		
		subtask = Task(self.id, self.tree)
		self.addSubtask(subtask)
			
	def _clicked(self, event):
		""" Callback appelé lorsqu'on click sur la tâche sans Shift appuyé"""
		
		self.tree.selectTask(self.id, False)

	
	def _alsoClicked(self, event):
		""" Callback appelé lorsqu'on click sur la tâche avec Shift appuyé"""
		
		self.tree.selectTask(self.id, True)

		
		
	def select(self, bool):
		"""Cette fonction permet de selectionner/deselectionner la tâche en cours"""
		
		self.selected = bool
		if bool:
			self.mainTask["bg"] = "red"
		else:
			self.mainTask["bg"] = "grey"
		
		